<?php

namespace Litecms\Franchise\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Franchise\Http\Requests\FranchiseRequest;
use Litecms\Franchise\Interfaces\FranchiseRepositoryInterface;
use Litecms\Franchise\Models\Franchise;

/**
 * Resource controller class for franchise.
 */
class FranchiseResourceController extends BaseController
{

    /**
     * Initialize franchise resource controller.
     *
     * @param type FranchiseRepositoryInterface $franchise
     *
     * @return null
     */
    public function __construct(FranchiseRepositoryInterface $franchise)
    {
        parent::__construct();
        $this->repository = $franchise;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Franchise\Repositories\Criteria\FranchiseResourceCriteria::class);
    }

    /**
     * Display a list of franchise.
     *
     * @return Response
     */
    public function index(FranchiseRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Franchise\Repositories\Presenter\FranchisePresenter::class)
                ->$function();
        }

        $franchise_s = $this->repository->paginate();

        return $this->response->title(trans('franchise::franchise.names'))
            ->view('franchise::franchise.index', true)
            ->data(compact('franchise_s'))
            ->output();
    }

    /**
     * Display franchise.
     *
     * @param Request $request
     * @param Model   $franchise
     *
     * @return Response
     */
    public function show(FranchiseRequest $request, Franchise $franchise)
    {

        if ($franchise->exists) {
            $view = 'franchise::franchise.show';
        } else {
            $view = 'franchise::franchise.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('franchise::franchise.name'))
            ->data(compact('franchise'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new franchise.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FranchiseRequest $request)
    {

        $franchise = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('franchise::franchise.name')) 
            ->view('franchise::franchise.create', true) 
            ->data(compact('franchise'))
            ->output();
    }

    /**
     * Create new franchise.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FranchiseRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $franchise                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('franchise::franchise.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('franchise/franchise/' . $franchise->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/franchise/franchise'))
                ->redirect();
        }

    }

    /**
     * Show franchise for editing.
     *
     * @param Request $request
     * @param Model   $franchise
     *
     * @return Response
     */
    public function edit(FranchiseRequest $request, Franchise $franchise)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('franchise::franchise.name'))
            ->view('franchise::franchise.edit', true)
            ->data(compact('franchise'))
            ->output();
    }

    /**
     * Update the franchise.
     *
     * @param Request $request
     * @param Model   $franchise
     *
     * @return Response
     */
    public function update(FranchiseRequest $request, Franchise $franchise)
    {
        try {
            $attributes = $request->all();

            $franchise->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('franchise::franchise.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('franchise/franchise/' . $franchise->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('franchise/franchise/' . $franchise->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the franchise.
     *
     * @param Model   $franchise
     *
     * @return Response
     */
    public function destroy(FranchiseRequest $request, Franchise $franchise)
    {
        try {

            $franchise->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('franchise::franchise.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('franchise/franchise/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('franchise/franchise/' . $franchise->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple franchise.
     *
     * @param Model   $franchise
     *
     * @return Response
     */
    public function delete(FranchiseRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('franchise::franchise.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('franchise/franchise'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/franchise/franchise'))
                ->redirect();
        }

    }

    /**
     * Restore deleted franchise_s.
     *
     * @param Model   $franchise
     *
     * @return Response
     */
    public function restore(FranchiseRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('franchise::franchise.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/franchise/franchise'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/franchise/franchise/'))
                ->redirect();
        }

    }

}
